/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.*
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.model.MultiPickerRequestParams
import com.kakao.sdk.partner.friend.model.PickerRequestParams
import com.kakao.sdk.partner.friend.model.SelectedUser
import com.kakao.sdk.partner.friend.model.SelectedUsers
import com.kakao.sdk.partner.friend.view.FriendPickerActivity
import com.kakao.sdk.partner.friend.view.FriendPopupPickerActivity

/**
 * @suppress
 */
internal class FriendPickerManager(
    private val isSingle: Boolean = false,
    private val isPopup: Boolean = false
) {
    fun startActivity(
        context: Context, param: Parcelable,
        callback: (selectedUsers: SelectedUsers?, error: Throwable?) -> Unit
    ) {
        if (!isParameterCorrect(param, callback)) {
            return
        }

        val requestParam = param as PickerRequestParams
        Repository.loadFriendsData(
            requestParam.pickerFriendOrder,
            requestParam.pickerFriendType ?: Constants.DEFAULT_PICKER_FRIEND_TYPE,
            requestParam.pickerFriendFilter ?: Constants.DEFAULT_PICKER_FRIEND_FILTER,
            requestParam.countryCodes
        ) { error ->
            if (error != null) {
                Repository.clearData()
                callback(null, error)
            } else {
                val activity =
                    if (isPopup) FriendPopupPickerActivity::class.java else FriendPickerActivity::class.java
                val intent =
                    Intent(context, activity)
                        .putExtra(Constants.KEY_BUNDLE, Bundle().apply {
                            putBoolean(Constants.KEY_IS_SINGLE, isSingle)
                            putBoolean(Constants.KEY_IS_POPUP, isPopup)
                            putParcelable(Constants.KEY_RESULT_RECEIVER, resultReceiver(callback))
                            putParcelable(Constants.KEY_PARAM, param)
                        })
                        .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                context.startActivity(intent)
            }
        }
    }

    @JvmSynthetic
    internal fun resultReceiver(
        callback: (selectedUsers: SelectedUsers?, error: Throwable?) -> Unit
    ): ResultReceiver =
        object : ResultReceiver(Handler(Looper.getMainLooper())) {
            override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
                SdkLog.d("***** Friend Picker RESULT: $resultData")
                if (resultCode == Activity.RESULT_OK) {
                    val friends =
                        resultData?.getParcelableArrayList<SelectedUser>(Constants.KEY_SELECTED_FRIENDS)

                    callback(friends?.let { SelectedUsers(it) }, null)
                } else if (resultCode == Activity.RESULT_CANCELED) {
                    val error =
                        resultData?.getSerializable(Constants.KEY_EXCEPTION) as KakaoSdkError
                    callback(null, error)
                } else {
                    val error = IllegalArgumentException("Unknown resultCode")
                    callback(null, error)
                }
                Repository.clearData() // Repository 클래스에 있는 데이터 제거
            }
        }

    private fun isParameterCorrect(
        param: Parcelable,
        callback: (selectedUsers: SelectedUsers?, error: Throwable?) -> Unit
    ): Boolean {
        val multiParam = if (param is MultiPickerRequestParams) {
            param
        } else {
            return true
        }

        val minPickableCount = multiParam.minPickableCount ?: Constants.DEFAULT_MIN_PICKABLE_COUNT
        val maxPickableCount = multiParam.maxPickableCount ?: Constants.DEFAULT_MAX_PICKABLE_COUNT
        val exception = when {
            minPickableCount < Constants.MULTI_PICKER_MIN_PICKABLE_COUNT ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter minPickableCount must be greater than 1."
                )

            maxPickableCount > Constants.MULTI_PICKER_MAX_PICKABLE_COUNT ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter maxPickableCount must be 100 or less."
                )

            minPickableCount > maxPickableCount ->
                ClientError(
                    ClientErrorCause.BadParameter,
                    "Parameter maxPickableCount is greater than or equal to parameter minPickableCount"
                )

            else -> null
        }

        exception?.let {
            resultReceiver(callback).send(Activity.RESULT_CANCELED, Bundle().apply {
                putSerializable(Constants.KEY_EXCEPTION, it)
            })
            return false
        }
        return true
    }
}